% Post treatment of FINELG output data - Ductility considered

% Conducted as part of master thesis relating to the development of an
% innovative model for the precise characterization of steel and composite
% joints

% MATHIEU Julien

close all
clear
clc

%% Results extraction

% Extraction of shear resistance data from excel file
data_all = xlsread('Data.xlsx');

% First line/column of shear resistance table
First_line_table = 2;
First_column_table = 16;

% Number of rows in tension/compression
n_rows_t = data_all(10,1);
n_rows_c = data_all(11,1);
n_rows_tot = n_rows_t+n_rows_c;

% Shear resistance data
for i = First_line_table : First_line_table+n_rows_t-1
    
        shear_resistance(i-(First_line_table-1),1) = data_all(i,First_column_table+1);
        shear_resistance(i-(First_line_table-1),2) = data_all(i,First_column_table+2);
        shear_resistance(i-(First_line_table-1),3) = data_all(i,First_column_table+4);
        
end

% Name of .TPS file
Name = 'Input_Finelg';
Ext = '.TPS';
FileName = (fullfile([Name, Ext]));

% Name of loads/max displacment file
Name_txt = 'Loads_Dep';
Ext_txt = '.txt';
FileName_txt = (fullfile([Name_txt, Ext_txt]));

% Name of displacements file
Name_Delta = 'Input_Finelg';
Ext_Delta = '.DE3';
FileName_Delta = (fullfile([Name_Delta, Ext_Delta]));


%% MN diagram

% Number of .TPS/loads files
if exist(FileName,'file')
    
    Dir = dir(fullfile([Name, '*', Ext]));
    nb_files = size(Dir,1);
    
else
    
    nb_files = 1;
    
end

% Values of final load multipliers
for i = 1:nb_files
    
    % Name of output files
    if i > 1
        
        FileName = fullfile([Name, sprintf('%d', i-1), Ext]);
        FileName_txt = fullfile([Name_txt, sprintf('%d', i-1), Ext_txt]);
        FileName_Delta = fullfile([Name, sprintf('%d', i-1), Ext_Delta]);
        
    end
    
    % Extraction of results
    % TPS
    fid = fopen(FileName,'r');
    
    Results = textscan(fid,repmat('%s',[1,25]),'Delimiter','*');
    
    fclose(fid);
    
    % Loads and displacement
    fid = fopen(FileName_txt,'r');
    
    Loads_Delta = textscan(fid,'%f%f%f');
    
    fclose(fid);
    
    % Convertion to numbers
    Mult = Results{1,4};
    Ajust = Results{1,3};
    Step = Results{1,2};
    
    Ajust(cellfun(@isempty,Ajust)) = [];
    Mult(cellfun(@isempty,Mult)) = [];
    Step(cellfun(@isempty,Step)) = [];
    
    Ajust = str2double(Ajust(2:end,:));
    Mult = cell2mat(Mult(2:end,:));
    Increment = Mult(:,7:end-2);
    Mult = Mult(:,1:end-9);
    Step = str2double(Step(2:end,:));
    
    for j = 1:size(Mult,1)
        
        Mult_all(j,i) = str2double(sscanf(Mult(j,:),'%s'));
        
    end
    
    Loads_Delta = cell2mat(Loads_Delta);
    % Imposed loads
    Loads_all(i,1) = Loads_Delta(1,1);
    Loads_all(i,2) = Loads_Delta(1,2);
    Loads_all(i,3) = Loads_Delta(1,3);
    
    % Maximum displacement
    Delta_max = Loads_Delta(2:end,1:2);
    
    % Extraction of displacement of all nodes at each step
    [DEP,NENR,LIB,NOMAX,FREQ] = lect_DE3(FileName_Delta,'C:');
    
    n = 1;
    Delta_all = zeros(size(DEP,1)/3,size(DEP,2));
    
    for j = 1:size(DEP,1)
        if j == 3*(n-1)+1
            
            Delta_all(n,:) = DEP(j,:);
            n = n+1;
            
        end
    end
    
    % Displacement of each component located in the last zone
    Delta_spring = zeros(size(Delta_max,1),size(Delta_all,2));
    for j = 1:size(Delta_max,1)
        for k = 1:size(Delta_all,1)
            if k == Delta_max(j,2)
                Delta_spring(j,:) = Delta_all(k,:)-Delta_all(k-1,:);
            end
        end
    end
    
    % Negative displacement in components in tension = 0
    % Positive displacement in components in compression = 0
    for j = 1:size(Delta_spring,1)
        for k = 1:size(Delta_spring,2)
            if Delta_max(j,1) < 0 && Delta_spring(j,k) > 0
                
                Delta_spring(j,k) = 0;
                
            end
            if Delta_max(j,1) > 0 && Delta_spring(j,k) < 0
                
                Delta_spring(j,k) = 0;
                
            end
        end
    end
    
    % Verification of the displacements
     n_steps = nnz(any(Mult_all(:,i),2));
    leave = 0;
    for j = 1:size(Delta_spring,2)
        for k = 1:size(Delta_spring,1)
            if abs(Delta_spring(k,j)) > abs(Delta_max(k,1))
                
                n_steps = nnz(any(Mult_all(:,i),2))-(size(Delta_all,2)-(j-1));
                n_steps_delta = j-1;
                leave = 1;
                
            end
            if leave == 1
                break
            end
        end
        if leave == 1
            break
        end
    end
    
    % Value of the load multiplier

    Multip(i,1) = Mult_all(n_steps,i);

    % Value of bending moment and axial force at failure
    MN(i,1:2) = Loads_all(i,1:2)*Multip(i,1);
    
    % Final step number

    fprintf('%s %s %s %d %s\n','Final step of',FileName,'=',Step(end,1),'(max = 40)')
    fprintf('%s %s %s %s\n','Final increment of',FileName,'=',Increment(end,:))
    
    % Shear resistance
    n = 0;
    if n_steps == nnz(any(Mult_all(:,i),2))
        for j = 1:n_rows_tot
            if Delta_max(j,1) < 0 && Delta_spring(j,end) <= 0
                n = n+1;
            elseif Delta_max(j,1) >= 0 && Delta_spring(j,end) == 0 
                
                shear_resistance_bolts(j-n,1) = shear_resistance(j-n,1);
                
            elseif Delta_max(j,1) >= 0 && Delta_spring(j,end) > 0 
                
                shear_resistance_bolts(j-n,1) = shear_resistance(j-n,1)*0.29;
                
            end
        end
    else
        for j = 1:n_rows_tot
            if Delta_max(j,1) < 0 && Delta_spring(j,n_steps_delta) <= 0
                n = n+1;
                continue
            elseif Delta_max(j,1) >= 0 && Delta_spring(j,n_steps_delta) == 0 
                
                shear_resistance_bolts(j-n,1) = shear_resistance(j-n,1);
                
            elseif Delta_max(j,1) >= 0 && Delta_spring(j,n_steps_delta) > 0 
                
                shear_resistance_bolts(j-n,1) = shear_resistance(j-n,1)*0.29;
                
            end
        end
    end
    
    for j = 1:n_rows_t  
        if shear_resistance_bolts(j,1) <= shear_resistance(j,2)
            resistance_bolts(j,1) = shear_resistance_bolts(j,1);
        else
            resistance_bolts(j,1) = shear_resistance(j,2);
        end
    end

    Total_resistance = sum(resistance_bolts);

    for j = 1:sum(not(isnan(shear_resistance(:,3))))
        if Total_resistance >= shear_resistance(j,3)
        
            Total_resistance = shear_resistance(j,3);  
        
        else 
            continue
        end
    end
    
    if Loads_Delta(1,3) <= Total_resistance
    
        fprintf('%s %d%s %s %d%s %s\n\n','Shear resistance :',Loads_Delta(1,3),'N','<=',Total_resistance,'N','OK')

    else
    
        fprintf('%s %d%s %s %d%s %s\n\n','Shear resistance :',Loads_Delta(1,3),'N','>',Total_resistance,'N','!NOT OK!')
    
    end
end

% Values of the four peaks of the MN diagram
[Mpos_max,loc(1,1)] = max(MN(:,1));
[Npos_max,loc(2,1)] = max(MN(:,2));
[Mneg_max,loc(3,1)] = min(MN(:,1));
[Nneg_max,loc(4,1)] = min(MN(:,2));

MN1_max = MN(loc(1,1),:);
MN2_max = MN(loc(2,1),:);
MN3_max = MN(loc(3,1),:);
MN4_max = MN(loc(4,1),:);


% Division of the MN diagram in four zones
n1 = 0;
n2 = 0;
n3 = 0;
n4 = 0;
for i = 1:size(MN,1)
    if MN(i,1) <= MN1_max(1,1) && MN(i,2) >= MN1_max(1,2) && MN(i,1) > MN2_max(1,1) && MN(i,2) < MN2_max(1,2)
        
        n1 = n1+1;
        M1(n1,1) = MN(i,1)/10^6;
        N1(n1,1) = MN(i,2)/10^3;
        
    elseif MN(i,1) <= MN2_max(1,1) && MN(i,2) <= MN2_max(1,2) && MN(i,1) > MN3_max(1,1) && MN(i,2) > MN3_max(1,2)
        
        n2 = n2+1;
        M2(n2,1) = MN(i,1)/10^6;
        N2(n2,1) = MN(i,2)/10^3;
        
    elseif MN(i,1) >= MN3_max(1,1) && MN(i,2) <= MN3_max(1,2) && MN(i,1) < MN4_max(1,1) && MN(i,2) > MN4_max(1,2)
        
        n3 = n3+1;
        M3(n3,1) = MN(i,1)/10^6;
        N3(n3,1) = MN(i,2)/10^3;
        
    elseif MN(i,1) >= MN4_max(1,1) && MN(i,2) >= MN4_max(1,2) && MN(i,1) < MN1_max(1,1) && MN(i,2) < MN1_max(1,2)
        
        n4 = n4+1;
        M4(n4,1) = MN(i,1)/10^6;
        N4(n4,1) = MN(i,2)/10^3;
        
    end
end

% Sorting of the MN values in the considered zone
M1 = sort(M1,'descend');
N1 = sort(N1,'ascend');
M2 = sort(M2,'descend');
N2 = sort(N2,'descend');
M3 = sort(M3,'ascend');
N3 = sort(N3,'descend');
M4 = sort(M4,'ascend');
N4 = sort(N4,'ascend');

% Plot
M = [M1' M2' M3' M4' M1(1,1)'];
N = [N1' N2' N3' N4' N1(1,1)'];

plot(M,N,'red')
hold on
Analytique;
grid on
